# frozen_string_literal: true

# Legende
# Les script a plusieurs parties indique que les bash (Batch dans l'app)
# peuvent s'initier avec differents types de source
# Le besoin est donc d'isoler la logique associee au modele actuel
# ne pas oublier d'indexer les champs simples

module Hermes
  # Lessor model
  class Lessor < Hermes::Entity
    hermes_tools 'handle_dso'

    field :api_id, type: String
    field :rpls_id, type: String
    field :affiliate, type: String

    has_one :contact_group, class_name: 'Hermes::ContactGroup'

    #TODO: le setteur(ou le getteur)de lessor_objective ne fonctionne pas
    field :lessor_objective_ids, type: Array, default: []
    has_many :lessor_objectives, class_name: 'Hermes::LessorObjective'
    has_many :agencies, class_name: 'Hermes::Entity', foreign_key: :id, primary_key: :lessor_objective_ids

    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id

    def instance_values(instance)
      instance.send(case instance.class.name.demodulize
                    when 'Tagset'
                      'superfields'
                    when 'HousingOffer'
                      'attributes'
                    end)
    end

    def lessor_objective_ids=(arr)
      objective_ids_to_unlink = lessor_objective_ids - (arr || [])
      now = DateTime.current
      Hermes::LessorObjective.where(:id.in => objective_ids_to_unlink).set(lessor_id: nil, update_at: now)
      Hermes::LessorObjective.where(:id.in => arr).set(lessor_id: id, updated_at: now)
      super(arr)
    end

    def lessor_objective_ids
      lessor_objectives.pluck(:id)
    end

    def handle_objectives_for(housing_offer)
      offer_fields = instance_values(housing_offer)
      query = offer_fields.select do |key, val|
        filter_included = Hermes::LessorObjective.filter_keys.include?(key.to_sym)
        valid_val = val.present? || val.is_a?(FalseClass)
        filter_included && valid_val
      end
      query[:$or] = [
        { present_typologies: offer_fields['typology'] },
        { present_typologies: 'all' }
      ]

      lessor_objectives.actives.where(query).each do |objective|
        objective.increment_realized_for(offer_fields['typology'])
      end
    end

    def sum_objectives_for(objectives, realized_or_target)
      return unless objectives.present?

      objectives.inject(0) do |sum, objective|
        sum + objective.sum_by_typologies_for(realized_or_target)
      end
    end

    def housing_actual_flow_number
      sum_objectives_for(lessor_objectives.actives, 'realized')
    end

    def housing_targeted_flow_number
      sum_objectives_for(lessor_objectives.actives, 'target')
    end

    def new_housing_actual_flow_number
      sum_objectives_for(lessor_objectives.actives.with_new, 'realized')
    end

    def new_housing_targeted_flow_number
      sum_objectives_for(lessor_objectives.actives.with_new, 'target')
    end
  end
end
